@@ -26,6 +26,7 @@ import util
 import types
 import os
 
+
 class ServerLayer(object):
 
     """A layer that starts/stops an subprocess and optionally checks
@@ -33,7 +34,8 @@ class ServerLayer(object):
 
     __bases__ = ()
 
-    def __init__(self, name, servers=[], start_cmd=None, subprocess_args={}):
+    def __init__(self, name, servers=[], start_cmd=None, subprocess_args={},
+                 stdout=None, stderr=None):
         self.__name__ = name
         self.servers = []
         self.start_cmd = start_cmd
@@ -41,6 +43,12 @@ class ServerLayer(object):
         for server in servers:
             host, port = server.split(':')
             self.servers.append((host, int(port)))
+        self.stdout = None
+        self.stderr = None
+        if stdout:
+            self.stdout = self.getFileObject(stdout)
+        if stderr:
+            self.stderr = self.getFileObject(stderr, 'stderr')
 
     def start(self):
         assert self.start_cmd, 'No start command defined'
@@ -71,11 +79,18 @@ class ServerLayer(object):
     def stop(self):
         self.process.kill()
         self.process.wait()
+        if self.stdout and not self.stdout.closed:
+            self.stdout.close()
+        if self.stderr and not self.stderr.closed:
+            self.stderr.close()
 
     def setUp(self):
+        if self.stdout:
+            self.stdout = self._reopen(self.stdout)
+        if self.stderr:
+            self.stderr = self._reopen(self.stderr, 'stderr')
         self.start()
 
-
     def tearDown(self):
         self.stop()
         to_stop = deque(self.servers)
@@ -89,4 +104,24 @@ class ServerLayer(object):
             else:
                 logging.info('Server stopped %s:%s', *server)
 
+    def getFileObject(self, path, ident='stdout'):
+        """ checks if the object is a file path or already a file object
+            If the object is a file path, a file object for the given path gets
+            returned.
+            If the path is a directory, also a file object gets created at the
+            given path """
+        if isinstance(path, file):
+            return path
+        elif isinstance(path, str):
+            if os.path.isdir(path):
+                path = os.path.join(path, '%s_%s' % (self.__name__, ident))
+            return open(path, 'w+')
+        return None
 
+    def _reopen(self, f, ident='stdout'):
+        """ checks if a file is closed and reopen it if so.
+            Also the file gets added to the subprocess_args. """
+        if f.closed:
+            f = open(f.name, 'w+')
+        self.subprocess_args[ident] = f
+        return f
